@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -481,7 +481,7 @@ gst_plugin_register_func (GstPlugin * plugin, const GstPluginDesc * desc,
   if (!gst_plugin_check_version (desc->major_version, desc->minor_version)) {
     if (GST_CAT_DEFAULT)
       GST_WARNING ("plugin \"%s\" has incompatible version, not loading",
-          plugin->filename);
+          GST_STR_NULL (plugin->filename));
     return NULL;
   }
 
@@ -489,14 +489,14 @@ gst_plugin_register_func (GstPlugin * plugin, const GstPluginDesc * desc,
       !desc->package || !desc->origin) {
     if (GST_CAT_DEFAULT)
       GST_WARNING ("plugin \"%s\" has incorrect GstPluginDesc, not loading",
-          plugin->filename);
+          GST_STR_NULL (plugin->filename));
     return NULL;
   }
 
   if (!gst_plugin_check_license (desc->license)) {
     if (GST_CAT_DEFAULT)
       GST_WARNING ("plugin \"%s\" has invalid license \"%s\", not loading",
-          plugin->filename, desc->license);
+          GST_STR_NULL (plugin->filename), desc->license);
     return NULL;
   }
 
@@ -514,13 +514,15 @@ gst_plugin_register_func (GstPlugin * plugin, const GstPluginDesc * desc,
   if (user_data) {
     if (!(((GstPluginInitFullFunc) (desc->plugin_init)) (plugin, user_data))) {
       if (GST_CAT_DEFAULT)
-        GST_WARNING ("plugin \"%s\" failed to initialise", plugin->filename);
+        GST_WARNING ("plugin \"%s\" failed to initialise",
+            GST_STR_NULL (plugin->filename));
       return NULL;
     }
   } else {
     if (!((desc->plugin_init) (plugin))) {
       if (GST_CAT_DEFAULT)
-        GST_WARNING ("plugin \"%s\" failed to initialise", plugin->filename);
+        GST_WARNING ("plugin \"%s\" failed to initialise",
+            GST_STR_NULL (plugin->filename));
       return NULL;
     }
   }
@@ -1706,15 +1708,15 @@ gst_plugin_ext_dep_equals (GstPluginDep * dep, const gchar ** env_vars,
 /**
  * gst_plugin_add_dependency:
  * @plugin: a #GstPlugin
- * @env_vars: NULL-terminated array of environment variables affecting the
+ * @env_vars: (allow-none): NULL-terminated array of environment variables affecting the
  *     feature set of the plugin (e.g. an environment variable containing
  *     paths where to look for additional modules/plugins of a library),
  *     or NULL. Environment variable names may be followed by a path component
  *      which will be added to the content of the environment variable, e.g.
  *      "HOME/.mystuff/plugins".
- * @paths: NULL-terminated array of directories/paths where dependent files
- *     may be.
- * @names: NULL-terminated array of file names (or file name suffixes,
+ * @paths: (allow-none): NULL-terminated array of directories/paths where dependent files
+ *     may be, or NULL.
+ * @names: (allow-none): NULL-terminated array of file names (or file name suffixes,
  *     depending on @flags) to be used in combination with the paths from
  *     @paths and/or the paths extracted from the environment variables in
  *     @env_vars, or NULL.
@@ -1777,14 +1779,14 @@ gst_plugin_add_dependency (GstPlugin * plugin, const gchar ** env_vars,
 /**
  * gst_plugin_add_dependency_simple:
  * @plugin: the #GstPlugin
- * @env_vars: one or more environment variables (separated by ':', ';' or ','),
+ * @env_vars: (allow-none): one or more environment variables (separated by ':', ';' or ','),
  *      or NULL. Environment variable names may be followed by a path component
  *      which will be added to the content of the environment variable, e.g.
  *      "HOME/.mystuff/plugins:MYSTUFF_PLUGINS_PATH"
- * @paths: one ore more directory paths (separated by ':' or ';' or ','),
+ * @paths: (allow-none): one ore more directory paths (separated by ':' or ';' or ','),
  *      or NULL. Example: "/usr/lib/mystuff/plugins"
- * @names: one or more file names or file name suffixes (separated by commas),
- *   or NULL
+ * @names: (allow-none): one or more file names or file name suffixes (separated by commas),
+ *      or NULL
  * @flags: optional flags, or #GST_PLUGIN_DEPENDENCY_FLAG_NONE
  *
  * Make GStreamer aware of external dependencies which affect the feature
