@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -114,8 +114,9 @@ helper_find_peek (gpointer data, gint64 offset, guint size)
     for (walk = helper->buffers; walk; walk = walk->next) {
       GstMappedBuffer *bmp = (GstMappedBuffer *) walk->data;
       GstBuffer *buf = GST_BUFFER_CAST (bmp->buffer);
-      guint64 buf_offset = GST_BUFFER_OFFSET (buf);
-      guint buf_size = gst_buffer_get_size (buf);
+
+      buf_offset = GST_BUFFER_OFFSET (buf);
+      buf_size = bmp->map.size;
 
       /* buffers are kept sorted by end offset (highest first) in the list, so
        * at this point we save the current position and stop searching if 
@@ -175,7 +176,12 @@ helper_find_peek (gpointer data, gint64 offset, guint size)
   }
 
   bmap = g_slice_new0 (GstMappedBuffer);
+
+  if (!gst_buffer_map (buffer, &bmap->map, GST_MAP_READ))
+    goto map_failed;
+
   bmap->buffer = buffer;
+
   if (insert_pos) {
     helper->buffers = g_slist_insert_before (helper->buffers, insert_pos, bmap);
   } else {
@@ -186,8 +192,6 @@ helper_find_peek (gpointer data, gint64 offset, guint size)
     helper->buffers = g_slist_prepend (helper->buffers, bmap);
   }
 
-  gst_buffer_map (buffer, &bmap->map, GST_MAP_READ);
-
   return bmap->map.data;
 
 error:
@@ -195,6 +199,13 @@ error:
     GST_INFO ("typefind function returned: %s", gst_flow_get_name (ret));
     return NULL;
   }
+map_failed:
+  {
+    GST_ERROR ("map failed");
+    gst_buffer_unref (buffer);
+    g_slice_free (GstMappedBuffer, bmap);
+    return NULL;
+  }
 }
 
 /*
@@ -235,7 +246,7 @@ helper_find_get_length (gpointer data)
 /**
  * gst_type_find_helper_get_range:
  * @obj: A #GstObject that will be passed as first argument to @func
- * @parent: the parent of @obj or NULL
+ * @parent: (allow-none): the parent of @obj or NULL
  * @func: (scope call): A generic #GstTypeFindHelperGetRangeFunction that will
  *        be used to access data at random offsets when doing the typefinding
  * @size: The length in bytes
@@ -260,8 +271,6 @@ helper_find_get_length (gpointer data)
  *
  * Returns: (transfer full): the #GstCaps corresponding to the data stream.
  *     Returns #NULL if no #GstCaps matches the data stream.
- *
- * Since: 0.10.26
  */
 GstCaps *
 gst_type_find_helper_get_range (GstObject * obj, GstObject * parent,
@@ -471,7 +480,7 @@ buf_helper_find_suggest (gpointer data, GstTypeFindProbability probability,
 
 /**
  * gst_type_find_helper_for_data:
- * @obj: object doing the typefinding, or NULL (used for logging)
+ * @obj: (allow-none): object doing the typefinding, or NULL (used for logging)
  * @data: (in) (transfer none): a pointer with data to typefind
  * @size: (in) (transfer none): the size of @data
  * @prob: (out) (allow-none): location to store the probability of the found
@@ -543,7 +552,7 @@ gst_type_find_helper_for_data (GstObject * obj, const guint8 * data, gsize size,
 
 /**
  * gst_type_find_helper_for_buffer:
- * @obj: object doing the typefinding, or NULL (used for logging)
+ * @obj: (allow-none): object doing the typefinding, or NULL (used for logging)
  * @buf: (in) (transfer none): a #GstBuffer with data to typefind
  * @prob: (out) (allow-none): location to store the probability of the found
  *     caps, or #NULL
@@ -601,8 +610,6 @@ gst_type_find_helper_for_buffer (GstObject * obj, GstBuffer * buf,
  * Returns: (transfer full): the #GstCaps corresponding to @extension, or
  *     #NULL if no type could be found. The caller should free the caps
  *     returned with gst_caps_unref().
- * 
- * Since: 0.10.23
  */
 GstCaps *
 gst_type_find_helper_for_extension (GstObject * obj, const gchar * extension)
