@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 
@@ -120,6 +120,7 @@ gst_valve_init (GstValve * valve)
       GST_DEBUG_FUNCPTR (gst_valve_event));
   gst_pad_set_query_function (valve->srcpad,
       GST_DEBUG_FUNCPTR (gst_valve_query));
+  GST_PAD_SET_PROXY_CAPS (valve->srcpad);
   gst_element_add_pad (GST_ELEMENT (valve), valve->srcpad);
 
   valve->sinkpad = gst_pad_new_from_static_template (&sinktemplate, "sink");
@@ -130,6 +131,7 @@ gst_valve_init (GstValve * valve)
   gst_pad_set_query_function (valve->sinkpad,
       GST_DEBUG_FUNCPTR (gst_valve_query));
   GST_PAD_SET_PROXY_CAPS (valve->sinkpad);
+  GST_PAD_SET_PROXY_ALLOCATION (valve->sinkpad);
   gst_element_add_pad (GST_ELEMENT (valve), valve->sinkpad);
 }
 
@@ -223,12 +225,13 @@ static gboolean
 gst_valve_event (GstPad * pad, GstObject * parent, GstEvent * event)
 {
   GstValve *valve;
+  gboolean is_sticky = GST_EVENT_IS_STICKY (event);
   gboolean ret = TRUE;
 
   valve = GST_VALVE (parent);
 
   if (g_atomic_int_get (&valve->drop)) {
-    valve->need_repush_sticky |= GST_EVENT_IS_STICKY (event);
+    valve->need_repush_sticky |= is_sticky;
     gst_event_unref (event);
   } else {
     if (valve->need_repush_sticky)
@@ -240,7 +243,7 @@ gst_valve_event (GstPad * pad, GstObject * parent, GstEvent * event)
    * downwards.
    */
   if (g_atomic_int_get (&valve->drop)) {
-    valve->need_repush_sticky |= GST_EVENT_IS_STICKY (event);
+    valve->need_repush_sticky |= is_sticky;
     ret = TRUE;
   }
 
