@@ -16,18 +16,18 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include <unistd.h>
 
 #include <gst/check/gstcheck.h>
 
-#define UNDERRUN_LOCK() (g_mutex_lock (underrun_mutex))
-#define UNDERRUN_UNLOCK() (g_mutex_unlock (underrun_mutex))
-#define UNDERRUN_SIGNAL() (g_cond_signal (underrun_cond))
-#define UNDERRUN_WAIT() (g_cond_wait (underrun_cond, underrun_mutex))
+#define UNDERRUN_LOCK() (g_mutex_lock (&underrun_mutex))
+#define UNDERRUN_UNLOCK() (g_mutex_unlock (&underrun_mutex))
+#define UNDERRUN_SIGNAL() (g_cond_signal (&underrun_cond))
+#define UNDERRUN_WAIT() (g_cond_wait (&underrun_cond, &underrun_mutex))
 
 static GstElement *queue;
 
@@ -41,8 +41,8 @@ static gulong probe_id;
 
 static gint overrun_count;
 
-static GMutex *underrun_mutex;
-static GCond *underrun_cond;
+static GMutex underrun_mutex;
+static GCond underrun_cond;
 static gint underrun_count;
 
 static GList *events;
@@ -121,8 +121,6 @@ setup (void)
 
   overrun_count = 0;
 
-  underrun_mutex = g_mutex_new ();
-  underrun_cond = g_cond_new ();
   underrun_count = 0;
 
   events = NULL;
@@ -137,11 +135,6 @@ cleanup (void)
 
   drop_events ();
 
-  g_cond_free (underrun_cond);
-  underrun_cond = NULL;
-  g_mutex_free (underrun_mutex);
-  underrun_mutex = NULL;
-
   if (mysinkpad != NULL) {
     gst_pad_set_active (mysinkpad, FALSE);
     gst_check_teardown_sink_pad (queue);
@@ -230,6 +223,7 @@ GST_START_TEST (test_non_leaky_overrun)
   GstBuffer *buffer2;
   GstBuffer *buffer3;
   GstBuffer *buffer;
+  GstSegment segment;
 
   g_signal_connect (queue, "overrun",
       G_CALLBACK (queue_overrun_link_and_activate), NULL);
@@ -246,7 +240,9 @@ GST_START_TEST (test_non_leaky_overrun)
   UNDERRUN_WAIT ();
   UNDERRUN_UNLOCK ();
 
-  gst_pad_push_event (mysrcpad, gst_event_new_stream_start ());
+  gst_segment_init (&segment, GST_FORMAT_BYTES);
+  gst_pad_push_event (mysrcpad, gst_event_new_stream_start ("test"));
+  gst_pad_push_event (mysrcpad, gst_event_new_segment (&segment));
 
   fail_unless (underrun_count == 1);
   fail_unless (overrun_count == 0);
@@ -277,12 +273,12 @@ GST_START_TEST (test_non_leaky_overrun)
   fail_unless (overrun_count == 1);
 
   /* lock the check_mutex to block the first buffer pushed to mysinkpad */
-  g_mutex_lock (check_mutex);
+  g_mutex_lock (&check_mutex);
   /* now let the queue push all buffers */
   while (g_list_length (buffers) < 3) {
-    g_cond_wait (check_cond, check_mutex);
+    g_cond_wait (&check_cond, &check_mutex);
   }
-  g_mutex_unlock (check_mutex);
+  g_mutex_unlock (&check_mutex);
 
   fail_unless (overrun_count == 1);
   /* make sure we get the underrun signal before we check underrun_count */
@@ -323,6 +319,7 @@ GST_START_TEST (test_leaky_upstream)
   GstBuffer *buffer2;
   GstBuffer *buffer3;
   GstBuffer *buffer;
+  GstSegment segment;
 
   g_signal_connect (queue, "overrun", G_CALLBACK (queue_overrun), NULL);
   g_object_set (G_OBJECT (queue), "max-size-buffers", 2, "leaky", 1, NULL);
@@ -338,7 +335,9 @@ GST_START_TEST (test_leaky_upstream)
   UNDERRUN_WAIT ();
   UNDERRUN_UNLOCK ();
 
-  gst_pad_push_event (mysrcpad, gst_event_new_stream_start ());
+  gst_segment_init (&segment, GST_FORMAT_BYTES);
+  gst_pad_push_event (mysrcpad, gst_event_new_stream_start ("test"));
+  gst_pad_push_event (mysrcpad, gst_event_new_segment (&segment));
 
   fail_unless (overrun_count == 0);
   fail_unless (underrun_count == 1);
@@ -409,6 +408,7 @@ GST_START_TEST (test_leaky_downstream)
   GstBuffer *buffer2;
   GstBuffer *buffer3;
   GstBuffer *buffer;
+  GstSegment segment;
 
   g_signal_connect (queue, "overrun", G_CALLBACK (queue_overrun), NULL);
   g_object_set (G_OBJECT (queue), "max-size-buffers", 2, "leaky", 2, NULL);
@@ -424,7 +424,9 @@ GST_START_TEST (test_leaky_downstream)
   UNDERRUN_WAIT ();
   UNDERRUN_UNLOCK ();
 
-  gst_pad_push_event (mysrcpad, gst_event_new_stream_start ());
+  gst_segment_init (&segment, GST_FORMAT_BYTES);
+  gst_pad_push_event (mysrcpad, gst_event_new_stream_start ("test"));
+  gst_pad_push_event (mysrcpad, gst_event_new_segment (&segment));
 
   fail_unless (overrun_count == 0);
   fail_unless (underrun_count == 1);
@@ -490,6 +492,7 @@ GST_START_TEST (test_time_level)
 {
   GstBuffer *buffer = NULL;
   GstClockTime time;
+  GstSegment segment;
 
   g_signal_connect (queue, "overrun",
       G_CALLBACK (queue_overrun_link_and_activate), NULL);
@@ -507,7 +510,9 @@ GST_START_TEST (test_time_level)
   UNDERRUN_WAIT ();
   UNDERRUN_UNLOCK ();
 
-  gst_pad_push_event (mysrcpad, gst_event_new_stream_start ());
+  gst_segment_init (&segment, GST_FORMAT_BYTES);
+  gst_pad_push_event (mysrcpad, gst_event_new_stream_start ("test"));
+  gst_pad_push_event (mysrcpad, gst_event_new_segment (&segment));
 
   /* push buffer without duration */
   buffer = gst_buffer_new_and_alloc (4);
@@ -606,7 +611,7 @@ GST_START_TEST (test_time_level_task_not_started)
   UNDERRUN_WAIT ();
   UNDERRUN_UNLOCK ();
 
-  gst_pad_push_event (mysrcpad, gst_event_new_stream_start ());
+  gst_pad_push_event (mysrcpad, gst_event_new_stream_start ("test"));
 
   gst_segment_init (&segment, GST_FORMAT_TIME);
   segment.start = 1 * GST_SECOND;
@@ -637,6 +642,62 @@ GST_START_TEST (test_time_level_task_not_started)
 
 GST_END_TEST;
 
+GST_START_TEST (test_sticky_not_linked)
+{
+  GstEvent *event;
+  GstSegment segment;
+  gboolean ret;
+  GstFlowReturn flow_ret;
+
+  GST_DEBUG ("starting");
+
+  g_object_set (queue, "max-size-buffers", 1, NULL);
+
+  UNDERRUN_LOCK ();
+  fail_unless (gst_element_set_state (queue,
+          GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
+      "could not set to playing");
+  UNDERRUN_WAIT ();
+  UNDERRUN_UNLOCK ();
+
+  gst_pad_push_event (mysrcpad, gst_event_new_stream_start ("test"));
+
+  gst_segment_init (&segment, GST_FORMAT_TIME);
+  segment.start = 1 * GST_SECOND;
+  segment.stop = 5 * GST_SECOND;
+  segment.time = 0;
+  segment.position = 1 * GST_SECOND;
+
+  event = gst_event_new_segment (&segment);
+  ret = gst_pad_push_event (mysrcpad, event);
+  fail_unless (ret == TRUE);
+
+  /* the first few buffers can return OK as they are queued and gst_queue_loop
+   * is woken up, tries to push and sets ->srcresult to NOT_LINKED
+   */
+  flow_ret = GST_FLOW_OK;
+  while (flow_ret != GST_FLOW_NOT_LINKED)
+    flow_ret = gst_pad_push (mysrcpad, gst_buffer_new ());
+
+  /* send a new sticky event so that it will be pushed on the next gst_pad_push
+   */
+  event = gst_event_new_segment (&segment);
+  ret = gst_pad_push_event (mysrcpad, event);
+  fail_unless (ret == TRUE);
+
+  /* make sure that gst_queue_sink_event doesn't return FALSE if the queue is
+   * unlinked, as that would make gst_pad_push return ERROR
+   */
+  flow_ret = gst_pad_push (mysrcpad, gst_buffer_new ());
+  fail_unless_equals_int (flow_ret, GST_FLOW_NOT_LINKED);
+
+  GST_DEBUG ("stopping");
+  fail_unless (gst_element_set_state (queue,
+          GST_STATE_NULL) == GST_STATE_CHANGE_SUCCESS, "could not set to null");
+}
+
+GST_END_TEST;
+
 #if 0
 static gboolean
 event_equals_newsegment (GstEvent * event, gboolean update, gdouble rate,
@@ -763,6 +824,76 @@ GST_START_TEST (test_newsegment)
 GST_END_TEST;
 #endif
 
+static gpointer
+thread_func (gpointer data)
+{
+  int i = 0;
+  for (i = 0; i < 100; i++) {
+    GstCaps *caps;
+    GstQuery *query;
+    gboolean ok;
+    caps = gst_caps_new_any ();
+    query = gst_query_new_allocation (caps, FALSE);
+    ok = gst_pad_peer_query (mysrcpad, query);
+    gst_query_unref (query);
+    gst_caps_unref (caps);
+    query = NULL;
+    caps = NULL;
+
+    if (!ok)
+      break;
+  }
+
+  return NULL;
+}
+
+static gboolean query_func (GstPad * pad, GstObject * parent, GstQuery * query);
+
+static gboolean
+query_func (GstPad * pad, GstObject * parent, GstQuery * query)
+{
+
+  g_usleep (1000);
+  return TRUE;
+}
+
+GST_START_TEST (test_queries_while_flushing)
+{
+  GstEvent *event;
+  GThread *thread;
+  int i;
+
+  mysinkpad = gst_check_setup_sink_pad (queue, &sinktemplate);
+  gst_pad_set_query_function (mysinkpad, query_func);
+  gst_pad_set_active (mysinkpad, TRUE);
+
+  /* hard to reproduce, so just run it a few times in a row */
+  for (i = 0; i < 500; ++i) {
+    GST_DEBUG ("starting");
+    UNDERRUN_LOCK ();
+    fail_unless (gst_element_set_state (queue,
+            GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
+        "could not set to playing");
+    UNDERRUN_WAIT ();
+    UNDERRUN_UNLOCK ();
+
+    thread = g_thread_new ("deactivating thread", thread_func, NULL);
+    g_usleep (1000);
+
+    event = gst_event_new_flush_start ();
+    gst_pad_push_event (mysrcpad, event);
+
+    g_thread_join (thread);
+
+    GST_DEBUG ("stopping");
+    fail_unless (gst_element_set_state (queue,
+            GST_STATE_NULL) == GST_STATE_CHANGE_SUCCESS,
+        "could not set to null");
+  }
+}
+
+GST_END_TEST;
+
 static Suite *
 queue_suite (void)
 {
@@ -777,9 +908,11 @@ queue_suite (void)
   tcase_add_test (tc_chain, test_leaky_downstream);
   tcase_add_test (tc_chain, test_time_level);
   tcase_add_test (tc_chain, test_time_level_task_not_started);
+  tcase_add_test (tc_chain, test_queries_while_flushing);
 #if 0
   tcase_add_test (tc_chain, test_newsegment);
 #endif
+  tcase_add_test (tc_chain, test_sticky_not_linked);
 
   return s;
 }
